% GetUdir: function for computing an erfc()-based lookup table for
%          interpolating the direct space potential via cubic splines.
%
%   Arguments:
%   ewcoeff: the Ewald coefficient (see getalpha.m)
%   Lcut:    the direct space cutoff
%   spc:     the spacing of the lookup table (maximum 0.01 recommended)
%
%   Spacings of up to 0.04 will allow the interaction potential of two +1e
%   charges to be interpolated accurately to within 1.0e-7 kcal/mol for
%   distances of 4A of more (although the corresponding force will only be
%   accurate to within 5.0e-6 kcal/mol-A, a 50-fold increase in the error).
%   For very short-ranged interactions, smaller spacings (as low as 0.001)
%   should be used.  Because the lookup table is indexed by r, its total
%   size is 32*Lcut/spc bytes.
%
%   Spline interpolation from the lookup table returns the value of the
%   Ewald direct space potential; taking the negative of the spline
%   derivative returns the corresponding direct space force.
%
%   Usage:
%   [Utab] = GetUdir(ewcoeff, Lcut, spc) returns the lookup table, a
%     structure with the following attributes:
%     Utab.U: the spline table
%     Utab.spc: the spacing given by the argument spc
%     Utab.u: the potential (for reference only, not for interpolation)
%     Utab.du: the force (for reference only, not for interpolation)
%     Utab.ew: the Ewald coefficient (stored for later retrieval)
%     Utab.Uerr: maximum error in the interpolated potential at each bin
%     Utab.dUerr: maximum error in the interpolated force at each bin
function Utab = GetUdir(ewcoeff, Lcut, spc)

  % Activate error checking
  if (spc < 0.0),
    spc = abs(spc);
    chk = 1;
  else
    chk = 0;
  end
  
  ntab = ceil(Lcut/spc)+2;
  u = zeros(ntab,1);
  du = zeros(ntab,1);

  % Compute the potential at various points
  bfac = 2.0*ewcoeff/sqrt(pi);
  for i = 1:1:ntab,
    r = i*spc;
    invr = 1.0/r;
    ewr = r*ewcoeff;
    rer = erfc(ewr)*invr;
    u(i,1) = rer;
    du(i,1) = -(rer + bfac*exp(-ewr*ewr))*invr;
  end
  u = 332.0636*u;
  du = 332.0636*du*spc;

  % Store these original arrays for future reference
  Utab.u = u;
  Utab.du = du;
  Utab.ew = ewcoeff;

  % Now determine splines for the potential
  v = 1:ntab-1;
  vp = 2:ntab;
  Utab.U(v,1) = 2.0*(u(v,1)-u(vp,1)) + du(v,1) + du(vp,1);
  Utab.U(v,2) = 3.0*(u(vp,1)-u(v,1)) - 2.0*du(v,1) - du(vp,1);
  Utab.U(v,3) = du(v,1);
  Utab.U(v,4) = u(v,1);

  % Save the spacing for future reference
  Utab.spc = spc;

  % Compute the errors with this lookup table
  if (chk == 0),
    return
  end
  Utab.Uerr = zeros(ntab,1);
  Utab.dUerr = zeros(ntab,1);
  invspc = 1.0/spc;
  tpb = 332.0636*bfac;
  for i = 1:1:ntab-1,
    for dr = 0.4:0.1:0.6,
      r = (i + dr)*spc;
      invr = 1.0/r;
      ewr = r*ewcoeff;
      Uact = 332.0636*erfc(ewr)*invr;
      Fact = -(Uact + tpb*exp(-ewr*ewr))*invr;
      Fitp = (3.0*Utab.U(i,1)*dr^2 + 2.0*Utab.U(i,2)*dr + Utab.U(i,3))*invspc;
      Uitp = Utab.U(i,1)*dr^3 + Utab.U(i,2)*dr^2 + Utab.U(i,3)*dr + Utab.U(i,4);
      if (abs(Uact-Uitp) > Utab.Uerr(i,1)),
        Utab.Uerr(i,1) = abs(Uact-Uitp);
      end
      if (abs(Fact-Fitp) > Utab.dUerr(i,1)),
        Utab.dUerr(i,1) = abs(Fact-Fitp);
      end
    end
  end
end
