% IFrc: this function takes the result of the reciprocal space
%       calculation, the potential grid U, and uses the B-spline
%       coefficients and derivatives to compute the force on each particle.
%
%   Arguments:
%   B[x,y,z]:  the arrays of B-spline coefficients in the "x", "y", and
%              "z" grid dimensions (along each of the three unit cell
%              dimensions (see splcof.m, where the arrays are generated)
%   dB[x,y,z]: the arrays of B-spline derivatives (see splcof.m)
%   U:         the potential energy grid
%   gdim:      the unit cell dimensions
%   [i,j,k]qm: the charge map arrays (see splcof.m)
%
%   Note that the electrostatic potential energy of the system could also
%   be obtained in this procedure by simply computing Q.*U, or since Q is
%   not supplied as an argument the potential could be obtained from the
%   B-spline coefficients of all atoms.  However, in practice, the
%   electrostatic potential (and, perhaps more importantly, the reciprocal
%   space contribution to the virial tensor) is typically computed in
%   Fourier space while convoluting Q with the reciprocal space pair
%   potential.
%
%   In summary, the force on each atom is given by the derivative of the
%   charge grid Q with respect to the position of each individual atom
%   times the electrostatic potential, NOT the derivative of the
%   electrostatic potential grid U at the locations of each atom.
%
%   This function works for all types of unit cells.  If the unit cell
%   dimensions given in gdim have only three entries, the cell is assumed
%   to be orthorhombic; if there are six entries, then the cell angles will
%   be read from the final three.
%
%   Usage:
%   [f] = Ifrc(Bx, By, Bz, dBx, dBy, dBz, iqm, jqm, kqm, U, gdim)
%     returns the forces on all particles
function f = IFrc(Bx, By, Bz, dBx, dBy, dBz, iqm, jqm, kqm, U, gdim)

  % Recall the number of atoms, the interpolation order, and the grid
  % dimensions
  ordr1 = size(Bx);
  ordr2 = size(By);
  ordr3 = size(Bz);
  natom = ordr1(1,1);
  ordr1 = ordr1(1,2);
  ordr2 = ordr2(1,2);
  ordr3 = ordr3(1,2);
  ng = size(U);

  % Initialize the forces on each atom
  f = zeros(natom,3);
  ng1o = ng(1)-ordr1+2;
  ng2o = ng(2)-ordr2+2;
  ng3o = ng(3)-ordr3+2;
  dqmijkx = zeros(ordr1,ordr2,ordr3);
  dqmijky = zeros(ordr1,ordr2,ordr3);
  dqmijkz = zeros(ordr1,ordr2,ordr3);
  for h = 1:1:natom,
    ii = 1;
    f1 = 0.0;
    f2 = 0.0;
    f3 = 0.0;

    % See if we can do this in terms of operations MatLab likes, rather
    % than gobs of loops
    ih = iqm(h,1);
    jh = jqm(h,1);
    kh = kqm(h,1);
    if (ih < ng1o && jh < ng2o && kh < ng3o),
      Bxh = Bx(h,:);
      Byh = By(h,:);
      Bzh = Bz(h,:);
      dBzh = dBz(h,:);
      dqmijx = (dBx(h,:)'*Byh);
      dqmijy = (Bxh'*dBy(h,:));
      dqmijz = (Bxh'*Byh);
      for i = 1:1:ordr3,
        dqmijkx(:,:,i) = Bzh(i)*dqmijx;
        dqmijky(:,:,i) = Bzh(i)*dqmijy;
        dqmijkz(:,:,i) = dBzh(i)*dqmijz;
      end
      Ureg = U(iqm(h,:),jqm(h,:),kqm(h,:));
      f1 = -sum(sum(sum(dqmijkx.*Ureg)));
      f2 = -sum(sum(sum(dqmijky.*Ureg)));
      f3 = -sum(sum(sum(dqmijkz.*Ureg)));
    else
      for i = iqm(h,:),
        jj = 1;
        dqx = Bx(h,ii);
        tqx = dBx(h,ii);
        for j = jqm(h,:),
          kk = 1;
          tdqxy = tqx*By(h,jj);
          dtqxy = dqx*dBy(h,jj);
          ddqxy = dqx*By(h,jj);
          for k = kqm(h,:),

            % Forces in X, Y, and Z obtained from derivatives in X, Y, and Z
            ulm = U(i,j,k);
            f1 = f1 - tdqxy*Bz(h,kk)*ulm;
            f2 = f2 - dtqxy*Bz(h,kk)*ulm;
            f3 = f3 - ddqxy*dBz(h,kk)*ulm;
            kk = kk+1;
          end
          jj = jj+1;
        end
        ii = ii+1;
      end
    end
    f(h,:) = [ f1 f2 f3 ];
  end

  for i = 1:1:3,
    f(:,i) = f(:,i) * ng(i);
  end
  szg = size(gdim);
  if (szg(1,2) == 6),

    % If we have a non-orthorhombic unit cell, we must take into account the
    % unit cell geometry.  For some reason, we have to take the transpose of
    % the transformation matrix again.
    U = CompXfrm(gdim);
    Ut = U';
    f = (Ut*(f'))';
  else

    % In the orthorhombic unit cell, forces must simply be divided by the
    % grid spacing to get the proper units: kcal/mol-A
    invgdim = 1.0./gdim;
    for i = 1:1:3,
      f(:,i) = f(:,i) * invgdim(i);
    end
  end

  % Report any irregularities
  timg = sum(mean(abs(imag(f))));
  if (timg > 1.0e-8),
    fprintf('IFrc: Significant non-real forces (%.8f) encountered!\n', timg);
    return
  end

  % There may be some residual imaginary forces left over from the
  % convolution, on the order of the machine precision.  We just omit those
  % (unless, as stated above, they're not trivial).
  f = real(f);

return
