% LoadPrefac: loads the prefactors for the B grid, along with some
%             optimizations for the influence function that seem to be
%             helping the AMBER code achieve higher accuracy.
%
%   Arguments:
%   ordr:  the interpolation order
%   ng:    scalar integer, the size of the grid (in other functions, ng is
%          a 1 x 3 integer array, but here it is only a scalar)
%
%   The Fourier space relatives of B-Splines are Euler exponential splines;
%   this routine computes the squared magnitudes of Euler exponential
%   splines for use in generating the grid B (see pmeOrthoConvBC, for
%   example).  This routine includes a correction for the case when Euler
%   spline interpolation with an odd numbered order will fail, by setting
%   the value at the tail of the distribution to the average of the two
%   nearest neighbor points.  Without such a correction, these PME routines
%   would not be able to compute forces with interpolation orders of 3, 5,
%   ...
%
%   Usage:
%   [bv] = LoadPrefac(ordr, ng) returns the optimized prefactors to be
%     used in the computation of the grid B (see pmeOrthoConvBC.,m for
%     example)
%   [bv,bsp_mod] = LoadPrefac(...) returns the non-optimized prefactors as
%     well, for comparison purposes
function [bv,bsp_mod] = LoadPrefac(ordr, ng)

  % First, load up the coefficients for an on-point B-spline
  bsp_arr = zeros(ng,1);
  for i = 1:1:ordr-1,
    bsp_arr(i,1) = bspln(i,ordr);
  end

  % Allocate memory for the B grid lookup table
  bsp_mod = zeros(ng,1);

  twopi = 2.0*pi;
  for k = 1:1:ng,
    sum1 = 0.0;
    sum2 = 0.0;
    for j = 1:1:ng,
      arg = twopi*(k-1)*(j-1)/ng;
      sum1 = sum1 + bsp_arr(j,1)*cos(arg);
      sum2 = sum2 + bsp_arr(j,1)*sin(arg);
    end
    bsp_mod(k,1) = sum1*sum1 + sum2*sum2;
  end

  % Fix the ONE case where exponential Euler spline interpolation fails.
  for k = 1:1:ng,
    if (bsp_mod(k,1) < 1.0e-7),
      if (k > 1 && k < ng),
        bsp_mod(k,1) = 0.5*(bsp_mod(k-1,1) + bsp_mod(k+1));
      else
        
        % This should never happen, but if it does we set it very high so
        % that the inverse will be very small.
        bsp_mod(k,1) = 1.0e15;
      end
    end
  end
  
  % In the end, bsp_mod holds the pre-factors as Essmann would compute them
  bsp_mod = 1.0./bsp_mod;
  
  % Now, optimize the lambda coefficient
  bv = zeros(ng,1);
  ngc = ng/2;
  ordr2 = 2*ordr;
  for k = 1:1:ng,
    m = k-1;
    if (k > ngc),
      m = k - 1 - ng;
    end
    if (m == 0),
      lambda = 1.0;
    else
      gsum = GammaSum(m, ng, ordr);
      gsum2 = GammaSum(m, ng, ordr2);
      lambda = gsum/gsum2;      
    end
    bv(k,1) = lambda*lambda*bsp_mod(k,1);
  end

end
