% MapCharges: this function creates the charge grid base on arrays of
%             B-spline coefficients.
%
%   Arguments:
%   ng:          the desired grid dimensions
%   SP[x,y,z]:   the B-spline coefficients for the "x", "y", and "z"
%                dimensions (or the first, second, and third unit cell
%                dimensions in the case of a non-orthorhobmic unit cell)
%   [i,j,k]qmap: the grid indices to apply charge in the first, second, and
%                third dimensions
%
%   This function works for all types of unit cells.  Non-orthogonality of
%   unit cell lattice vectors affects work done in the determination of the
%   indices in [i,j,k]qmap, determination of the reciprocal space pair
%   potential, and interpolation of forces.
%
%   Usage:
%   [Q] = MapCharges(ng, SPx, SPy, SPz, iqmap, jqmap, kqmap) returns the
%     charge grid
function Q = MapCharges(ng, SPx, SPy, SPz, iqmap, jqmap, kqmap)

  % Determine the number of atoms and the interpolation order
  ordr1 = size(SPx);
  ordr2 = size(SPy);
  ordr3 = size(SPz);
  natom = ordr1(1,1);
  ordr1 = ordr1(1,2);
  ordr2 = ordr2(1,2);
  ordr3 = ordr3(1,2);
  
  % Initialize the charge grid
  Q = zeros(ng(1), ng(2), ng(3));

  % Loop over all atoms and over each relevant region
  ng1o = ng(1)-ordr1+2;
  ng2o = ng(2)-ordr2+2;
  ng3o = ng(3)-ordr3+2;
  qmijk = zeros(ordr1,ordr2,ordr3);
  for h = 1:1:natom,
    ii = 1;

    % See if we can do this in terms of operations MatLab likes, rather
    % than gobs of loops
    imp = iqmap(h,:);
    jmp = jqmap(h,:);
    kmp = kqmap(h,:);
    ih = imp(1);
    jh = jmp(1);
    kh = kmp(1);
    if (ih < ng1o && jh < ng2o && kh < ng3o),
      qmij = (SPx(h,:)'*SPy(h,:));
      SPzh = SPz(h,:);
      for i = 1:1:ordr3,
        qmijk(:,:,i) = SPzh(i)*qmij;
      end
      Q(imp,jmp,kmp) = Q(imp,jmp,kmp) + qmijk;
      continue
    end

    % Darn, gotta do a bunch of loops
    for i = iqmap(h,:),
      dqx = SPx(h,ii);
      jj = 1;
      for j = jqmap(h,:),
        dqxy = dqx*SPy(h,jj);
        kk = 1;
        for k = kqmap(h,:),
          Q(i,j,k) = Q(i,j,k) + dqxy*SPz(h,kk);
          kk = kk+1;
        end
        jj = jj+1;
      end
      ii = ii+1;
    end
  end
  
  % Finally, scale Q by Coulomb's constant
  Q = 332.0636*Q;
  
return
