% SubCellForce: this function takes as input a subcell structure and
%               returns the forces for all atoms due to interactions lying
%               within the active subcell region by the midpoint method.
%
%   Usage: in the context of the pmeDirectMidpointOrthog.m function
function G = SubCellForce(G, Utab, Lcut)

  % Find the active region
  Aorig = 0.5*G.bxl;
  Amax = 1.5*G.bxl;
  Allim = Aorig - 0.5*Lcut;
  Ahlim = Amax + 0.5*Lcut;
  Lcut2 = Lcut*Lcut;
  hLcut2 = 0.25*Lcut2;

  % Prune atoms that don't contribute to this subcell's force calculation.
  prn = zeros(G.natom,1);
  for i = 1:1:G.natom,

    % Everything outside half the cutoff is pruned
    for j = 1:1:3,
      if (G.crd(i,j) < Allim(j)),
        prn(i,1) = 1;
      end
      if (G.crd(i,j) > Ahlim(j)),
        prn(i,1) = 1;
      end
    end
    if (prn(i,1) == 1),
      continue
    end

    % Now sand down the edges
    npt = [ 0 0 0 ];
    for j = 1:1:3,
      if (G.crd(i,j) < Aorig(j)),
        npt(j) = -1;
      elseif (G.crd(i,j) > Amax(j)),
        npt(j) = 1;
      end
    end
    if (sum(abs(npt)) < 2),
      continue
    end

    % If we're still here, go through the list
    dr = [ 0.0 0.0 0.0 ];
    for j = 1:1:3,
      if (npt(j) == -1),
        dr(j) = Aorig(j) - G.crd(i,j);
      elseif (npt(j) == 1),
        dr(j) = G.crd(i,j) - Amax(j);
      end
    end
    r = sum(dr.*dr);
    if (r > hLcut2),
      prn(i,1) = 1;
    end
  end

  % Reduce the list
  h = 0;
  for i = 1:1:G.natom,
    if (prn(i,1) == 0),
      h = h+1;
      G.crd(h,:) = G.crd(i,:);
      G.q(h,1) = G.q(i,1);
      G.anum(h,1) = G.anum(i,1);
    end
  end
  G.natom = h;
  G.crd = G.crd(1:G.natom,:);
  G.q = G.q(1:G.natom,1);
  G.anum = G.anum(1:G.natom,1);

  % Find where things are located
  aloc = zeros(G.natom, 3);
  for i = 1:1:G.natom,
    for j = 1:1:3,
      if (G.crd(i,j) < Aorig(j)),
        aloc(i,j) = 0;
      elseif (G.crd(i,j) >= Amax(j)),
        aloc(i,j) = 2;
      else
        aloc(i,j) = 1;
      end
    end
  end

  % Initialize forces for this subcell
  frc = zeros(G.natom, 3);

  % Now that we've pruned some of the atoms, loop over them all
  uspc = 1.0/Utab.spc;
%  ewcoeff = 0.5/1.5;
%  bfac = 2.0*ewcoeff/sqrt(pi);

  nint = 0;
  for i = 1:1:G.natom,
    atmx = G.crd(i,1);
    atmy = G.crd(i,2);
    atmz = G.crd(i,3);
    atmq = G.q(i,1);
    ffx = 0.0;
    ffy = 0.0;
    ffz = 0.0;
    for j = i+1:1:G.natom,

      % Compute the range
      dx = G.crd(j,1) - atmx;
      dy = G.crd(j,2) - atmy;
      dz = G.crd(j,3) - atmz;
      r2 = dx*dx + dy*dy + dz*dz;
      if (r2 < Lcut2),

        % See if the interaction should be counted
        midx = atmx + 0.5*dx;
        if (midx < Aorig(1) || midx >= Amax(1)),
          continue;
        else
          midy = atmy + 0.5*dy;
          if (midy < Aorig(2) || midy >= Amax(2)),
            continue;
          else
            midz = atmz + 0.5*dz;
            if (midz < Aorig(3) || midz >= Amax(3)),
              continue
            end
          end
        end

        % Count the number of interactions
        nint = nint + 1;

        % Compute the magnitude of the force and divide by r again
	qq = atmq*G.q(j,1);
        r = sqrt(r2);
        rs = r*uspc;
        ir = floor(rs);
        dr = rs - ir;
        fmag = (3.0*Utab.U(ir,1)*dr^2 + 2.0*Utab.U(ir,2)*dr + Utab.U(ir,3));
        fmag = fmag*uspc*qq/r; 

        % Compute components of the force
        fx = fmag*dx;
        fy = fmag*dy;
        fz = fmag*dz;
        ffx = ffx + fx;
        ffy = ffy + fy;
        ffz = ffz + fz;
        frc(j,1) = frc(j,1) - fx;
        frc(j,2) = frc(j,2) - fy;
        frc(j,3) = frc(j,3) - fz;

      end
    end
    frc(i,:) = frc(i,:) + [ ffx ffy ffz ];
  end

  G.frc = frc;
  G.nint = nint;
end
