% TowerPlateFrc: this function takes as input a structure of subcells (each
%                a volume of space at least the direct space cutoff from
%                side to side) and computes interactions between all atoms
%                within that subcell and its tower and plate regions.
%
%   Usage: within the context of pmeDirectTowerPlate
function fdir = TowerPlateFrc(LC, ci, cj, ck, ncell, invU, Lcut, Ut, fdir)

  % Preliminary computations
  Lcut2 = Lcut*Lcut;
  uspc = 1.0/Ut.spc;
  ewcoeff = Ut.ew;
  bfac = 2.0*ewcoeff/sqrt(pi);
  tpb = 332.0636*bfac;
  
  % List all cells of the plate.  When this list is complete, the first
  % block is chopped off to avoid double-counting.  After that, the center
  % cell is the first cell in the plate list.
  plate = zeros(6,5);
  h = 1;
  for j = 0:1:1,
    cjm = cj + j;
    if (cjm > ncell(2)),
      cjm = cjm - ncell(2);
      dy = [ invU(1,2) invU(2,2) 0.0 ];
    elseif (cjm <= 0),
      cjm = cjm + ncell(2);
      dy = [ -invU(1,2) -invU(2,2) 0.0 ];
    else
      dy = [ 0.0 0.0 0.0 ];
    end
    for i = -1:1:1,
      cim = ci + i;
      if (cim > ncell(1)),
        cim = cim - ncell(1);
        dx = [ invU(1,1) 0.0 0.0 ];
      elseif (cim <= 0),
        cim = cim + ncell(1);
        dx = [ -invU(1,1) 0.0 0.0 ];
      else
        dx = [ 0.0 0.0 0.0 ];
      end
      plate(h,:) = [ cim cjm dx+dy ];
      h = h + 1;
    end
  end
  plate = plate(2:6,:);

  % Get all plate atoms in position
  for h = 1:1:5,
    hpi = plate(h,1);
    hpj = plate(h,2);
    dxy = plate(h,3:5);
    for m = 1:1:LC(hpi,hpj,ck).natom,
      LC(hpi,hpj,ck).crd(m,:) = LC(hpi,hpj,ck).crd(m,:) + dxy;
    end
  end

  % Loop over the tower cells
  for k = -1:1:1,

    % Determine the cell position and any necessary offset
    ckm = ck + k;
    if (ckm <= 0),
      ckm = ckm + ncell(3);
      dz = [ -invU(1,3) -invU(2,3) -invU(3,3) ];
    elseif (ckm > ncell(3)),
      ckm = ckm - ncell(3);
      dz = [ invU(1,3) invU(2,3) invU(3,3) ];
    else
      dz = [ 0.0 0.0 0.0 ];
    end

    % Make the necessary offset to tower atoms
    ntwatom = LC(ci,cj,ckm).natom; 
    if (ntwatom == 0),
      continue;
    end
    for m = 1:1:ntwatom,
      LC(ci,cj,ckm).crd(m,:) = LC(ci,cj,ckm).crd(m,:) + dz;
    end

    % Initialize force accumulator for this tower cell
    frcT = zeros(ntwatom, 3);

    % Loop over plate cells.  Skip the first plate cell (the center cell)
    % if the tower cell is beneath the plate.
    hmin = 1;
    if (k < 0),
      hmin = 2;
    end
    for h = hmin:1:5,
      hpi = plate(h,1);
      hpj = plate(h,2);

      % Initialize force accumulator for this plate cell
      nplatom = LC(hpi,hpj,ck).natom;
      if (nplatom == 0),
        continue;
      end
      frcP = zeros(nplatom, 3);

      nint = 0;
      tint = 0;

      % Loop over tower atoms
      icrd = LC(ci,cj,ckm).crd;
      jcrd = LC(hpi,hpj,ck).crd;
      iq = LC(ci,cj,ckm).q;
      jq = LC(hpi,hpj,ck).q;
      for i = 1:1:ntwatom,
        atmx = icrd(i,1);
        atmy = icrd(i,2);
        atmz = icrd(i,3);
        atmq = iq(i,1);

        % Initialize force accumulators for this atom
        ffx = 0.0;
        ffy = 0.0;
        ffz = 0.0;

        % Loop over plate atoms.  If we are dealing with center cell
        % interactions, set the loop limits accordingly.
        jmin = 1;
        if (k == 0 && h == 1),
          jmin = i+1;
        end
        for j = jmin:1:nplatom,
          
          % Determine the inter-atomic distance
          tx = jcrd(j,1) - atmx;
          ty = jcrd(j,2) - atmy;
          tz = jcrd(j,3) - atmz;
          r2 = tx*tx + ty*ty + tz*tz;
          if (r2 < Lcut2),

            % Compute the magnitude of the force and divide by r again
            qq = atmq*jq(j,1);
            r = sqrt(r2);

            % The short or long-ranged potential?
            if (r < 2.0),
              invr = 1.0/r;
              ewr = ewcoeff*r;
              fmag = -qq*(332.0636*erfc(ewr)*invr + tpb*exp(-ewr*ewr))*invr*invr;
            else
              rs = r*uspc;
              ir = floor(rs);
              dr = rs - ir;
              ttmp = Ut.U(ir,:);
              fmag = (3.0*ttmp(1)*dr^2 + 2.0*ttmp(2)*dr + ttmp(3));
              fmag = fmag*uspc*qq/r;
            end

            % Compute components of the force
            fx = fmag*tx;
            fy = fmag*ty;
            fz = fmag*tz;
            ffx = ffx + fx;
            ffy = ffy + fy;
            ffz = ffz + fz;
            frcP(j,1) = frcP(j,1) - fx;
            frcP(j,2) = frcP(j,2) - fy;
            frcP(j,3) = frcP(j,3) - fz;
          end
        end
        frcT(i,:) = frcT(i,:) + [ ffx ffy ffz ];
      end

      % Contribute to the plate cell forces
      for i = 1:1:nplatom,
        aan = LC(hpi,hpj,ck).anum(i,1);
        fdir(aan,:) = fdir(aan,:) + frcP(i,:);
      end
    end

    % Return tower atoms to their original positions and contribute to
    % tower cell forces.
    for m = 1:1:ntwatom,
      aan = LC(ci,cj,ckm).anum(m,:);
      fdir(aan,:) = fdir(aan,:) + frcT(m,:);
      LC(ci,cj,ckm).crd(m,:) = LC(ci,cj,ckm).crd(m,:) - dz;
    end
  end

  % Return all plate atoms to their original positions
  for h = 1:1:5,
    hpi = plate(h,1);
    hpj = plate(h,2);
    dxy = plate(h,3:5);
    for m = 1:1:LC(hpi,hpj,ck).natom,
      LC(hpi,hpj,ck).crd(m,:) = LC(hpi,hpj,ck).crd(m,:) - dxy;
    end
  end
end
