% pmeDirectBasicLoop: this function performs a basic loop over all atoms to
%                     compute the non-bonded direct space forces.  It is
%                     SLOW, but very simple to comprehend and thus serves
%                     as a good reference for designing other direct space
%                     interaction calculation schemes.
%
%   Arguments:
%   crdq:    the XYZ coordinates and charges for N atoms in an N x 4 matrix
%            (the number of atoms N is automatically detected)
%   excl:    the exclusions (not yet implemented, just feed the function a
%            scalar number as a placeholder for this argument)
%   gdim:    1 x 3 array, the unit cell dimensions (an orthorhombic unit
%            cell is assumed for the time being)
%   ewcoeff: the Ewald coefficient (I prefer to think in terms of the
%            Gaussian width, which is 0.5/ewcoeff, but for the purposes of
%            coding we work with this number)
%   Lcut:    the direct space cutoff (typical values are 8-9A)
%
%   This function works with all types of unit cells, hence there is a step
%   that slows the function down even further, requiring that all
%   displacements be computed by converting to box space, re-imaging, and
%   then converting back to real space.  Optimizations for orthorhombic
%   unit cells are not implemented.
%
%   Usage:
%   [f] = pmeDirectBasicLoo(crdq, excl, gdim, ewcoeff, Lcut) returns the
%     direct space forces on all atoms
function frc = pmeDirectBasicLoop(crdq, excl, gdim, ewcoeff, Lcut)

  % Shortcuts
  Lcut2 = Lcut*Lcut;
  Ut = GetUdir(ewcoeff, Lcut, 0.0001);
  uspc = 1.0/Ut.spc;
  
  % Get the box transformation matrices
  szg = size(gdim);
  szg = szg(1,2);
  if (szg == 3),
    gdim(4:6) = pi/2;
  end
  [U,invU] = CompXfrm(gdim);
  
  % Put atoms in box space and reimage
  bxcrd = (U*crdq(:,1:3)')';
  bxcrd = bxcrd - floor(bxcrd);
  natom = size(crdq);
  natom = natom(1,1);
    
  % Loop over all atoms
  frc = zeros(natom,3);
  for i = 1:1:natom-1,
    atmx = bxcrd(i,1);
    atmy = bxcrd(i,2);
    atmz = bxcrd(i,3);
    atmq = crdq(i,4);
    for j = i+1:1:natom,
      dx = bxcrd(j,1) - atmx;
      dy = bxcrd(j,2) - atmy;
      dz = bxcrd(j,3) - atmz;
      if (dx > 0.5),
        dx = dx - 1.0;
      elseif (dx < -0.5),
        dx = dx + 1.0;
      end
      if (dy > 0.5),
        dy = dy - 1.0;
      elseif (dy < -0.5),
        dy = dy + 1.0;
      end
      if (dz > 0.5),
        dz = dz - 1.0;
      elseif (dz < -0.5),
        dz = dz + 1.0;
      end
      ddr = (invU*[ dx ; dy ; dz ])';
      r2 = ddr(1)*ddr(1) + ddr(2)*ddr(2) + ddr(3)*ddr(3);
      if (r2 < Lcut2),
        qq = atmq*crdq(j,4);
        r = sqrt(r2);
        rs = r*uspc;
        ir = floor(rs);
        dr = rs - ir;
        fmag = (3.0*Ut.U(ir,1)*dr^2 + 2.0*Ut.U(ir,2)*dr + Ut.U(ir,3));
        fmag = fmag*uspc*qq/r;

        % Compute components of the force
        df = fmag*ddr;
        frc(i,:) = frc(i,:) + df;
        frc(j,:) = frc(j,:) - df;
      end
    end
    if (mod(i,1000) == 0),
      fprintf('Finished atom %5d\n', i);
    end
  end
end
