% pmeDirectMidpointOrthog: this function handles the direct space
%                          calculation for Smooth Particle Mesh Ewald, if
%                          the unit cell is orthorhombic.
%
%   Arguments:
%   crdq:    the XYZ coordinates and charges for N atoms in an N x 4 matrix
%            (the number of atoms N is automatically detected)
%   excl:    the exclusions (not yet implemented, just feed the function a
%            scalar number as a placeholder for this argument)
%   gdim:    1 x 3 array, the unit cell dimensions (an orthorhombic unit
%            cell is assumed for the time being)
%   ewcoeff: the Ewald coefficient (I prefer to think in terms of the
%            Gaussian width, which is 0.5/ewcoeff, but for the purposes of
%            coding we work with this number)
%   Lcut:    the direct space cutoff (typical values are 8-9A
%
%   In order to make this scale linearly in the number of atoms, a
%   simplified version of the Midpoint method is implemented.  The
%   basic concept is to decompose the space into sub-blocks, and then
%   figure out what interactions have their midpoints located within each
%   sub-block.  This works very well in terms of load-balancing for
%   parallel applications, as the geometry of the sub-block itself and the
%   direct space cutoff Lcut can be used to determine a minimal "import
%   region" containing all the atoms which could possibly participate in
%   interactions relevant to that particular sub-block (the import region
%   is the sub-block, inflated by 0.5*Lcut, rounded on all edges and
%   corners).
%
%   This routine sub-divides the unit cell into sub-blocks whose lengths
%   are AT LEAST Lcut, and then groups together sub-blocks (i,j,k),
%   (i+1,j,k), (i,j+1,k), ..., (i+1,j+1,k+1) (accounting for periodicity)
%   and treats that subset of atoms as the "import region" to be shipped
%   off to the function SubCellFrc which computes interactions for a region
%   that is the same volume as any one of the sub-blocks, but located at
%   the center of the octet of sub-blocks comprising the import region.
%
%   This function puts together a fairly extensive array of structures for
%   the various sub-cells, which can be passed back for later examination.
%
%   In a complete molecular dynamics implementation, the non-bonded
%   Lennard-Jones interactions would also be computed within this function,
%   but this implementation deals only with electrostatics.
%
%   Most important is to realize that, while the direct space sum does
%   appear to be very similar to the familiar double-loop over all charges
%   needed to compute the forces (or energy) for a system of point charges,
%   the direct space sum is technically a modification of the reciprocal
%   space sum, necessary to extract the interactions of point charges from
%   the interactions of the corresponding system of smoothed charges.  The
%   reciprocal space sum is "where it's at," where kqq/r dominates at long
%   range.  The direct space sum is more like an afterthought.
%
%   Usage:
%   [f] = pmeDirectOrthog(crdq, excl, gdim, ewcoeff, Lcut) returns the
%     direct space forces
%   [f,LC] = pmeDirectOrthog(...) returns the forces as well as the
%     organization of the atoms into separate subcells used in the
%     calculation
function [f,LC] = pmeDirectMidpointOrthog(crdq, excl, gdim, ewcoeff, Lcut)

  % First, figure out a reasonable decomposition
  nbx = floor(gdim/Lcut);
  bxl = gdim./nbx;
  invbxl = 1.0./bxl;
  invgdim = 1.0./gdim;
  for i = 1:1:nbx(1),
    for j = 1:1:nbx(2),
      for k = 1:1:nbx(3),
        LC(i,j,k).natom = 0;
        LC(i,j,k).orig = [ i-1 j-1 k-1 ].*bxl;
        LC(i,j,k).min = LC(i,j,k).orig;
      end
    end
  end

  % Partition atoms among subcells
  natom = size(crdq);
  natom = natom(1,1);
  fcrd = zeros(natom,3);
  reimcrd = zeros(natom,3);
  boxcrd = zeros(natom,3);
  for i = 1:1:natom,
    fcrd(i,:) = crdq(i,1:3).*invgdim;
    fcrd(i,:) = fcrd(i,:) - floor(fcrd(i,:));
    reimcrd(i,:) = fcrd(i,:).*gdim;
    boxcrd(i,:) = reimcrd(i,:).*invbxl + 1;
  end
  for h = 1:1:natom,
    i = floor(boxcrd(h,1));
    j = floor(boxcrd(h,2));
    k = floor(boxcrd(h,3));
    LC(i,j,k).natom = LC(i,j,k).natom+1;
    LC(i,j,k).nint = 0;
  end
  for i = 1:1:nbx(1),
    for j = 1:1:nbx(2),
      for k = 1:1:nbx(3),
        LC(i,j,k).q = zeros(LC(i,j,k).natom,1); 
        LC(i,j,k).anum = zeros(LC(i,j,k).natom,1); 
        LC(i,j,k).crd = zeros(LC(i,j,k).natom,3); 
        LC(i,j,k).frc = zeros(LC(i,j,k).natom,3); 
      end
    end
  end
  ccon = zeros(nbx(1), nbx(2), nbx(3));
  for h = 1:1:natom,
    i = floor(boxcrd(h,1));
    j = floor(boxcrd(h,2));
    k = floor(boxcrd(h,3));
    ccon(i,j,k) = ccon(i,j,k) + 1;
    LC(i,j,k).anum(ccon(i,j,k),1) = h;
    LC(i,j,k).crd(ccon(i,j,k),:) = crdq(h,1:3);
    LC(i,j,k).q(ccon(i,j,k),1) = crdq(h,4);
  end

  % Expand subcells
  for i = 1:1:nbx(1),
    if (i < nbx(1)), 
      ivec = [ i i+1 ];
      xoff = 0.0;
    else
      ivec = [ i 1 ];
      xoff = -bxl(1);
    end
    for j = 1:1:nbx(2),
      if (j < nbx(2)),
        jvec = [ j j+1 ];
        yoff = 0.0;
      else
        jvec = [ j 1 ];
        yoff = -bxl(2);
      end
      for k = 1:1:nbx(3),
        if (k < nbx(3)),
          kvec = [ k k+1 ];
          zoff = 0.0;
        else
          kvec = [ k 1 ];
          zoff = -bxl(3);
        end
        LC(i,j,k).Snatom = 0;
        gcrd = [];
        gq = [];
        ganum = [];
        for m = ivec,
          for n = jvec,
            for p = kvec,
              LC(i,j,k).Snatom = LC(i,j,k).Snatom + LC(m,n,p).natom;
              gcrd = [ gcrd ; LC(m,n,p).crd ];
              gq = [ gq ; LC(m,n,p).q ];
              ganum = [ ganum ; LC(m,n,p).anum ];
            end
          end
        end
        LC(i,j,k).bxl = bxl;
        for m = 1:1:3,
          gcrd(:,m) = gcrd(:,m) - LC(i,j,k).orig(m);
        end
        for m = 1:1:LC(i,j,k).Snatom,
          gcrd(m,:) = gcrd(m,:).*invgdim;
          gcrd(m,:) = (gcrd(m,:) - floor(gcrd(m,:))).*gdim;
        end
        LC(i,j,k).Scrd = gcrd;
        LC(i,j,k).Sq = gq;
        LC(i,j,k).Sanum = ganum;
      end
    end
  end
  for i = 1:1:nbx(1),
    for j = 1:1:nbx(2),
      for k = 1:1:nbx(3),
        LC(i,j,k).crd = LC(i,j,k).Scrd;
        LC(i,j,k).q = LC(i,j,k).Sq;
        LC(i,j,k).anum = LC(i,j,k).Sanum;
        LC(i,j,k).natom = LC(i,j,k).Snatom;
        clear LC(i,j,k).Scrd LC(i,j,k).Sq LC(i,j,k).Snatom;
      end
    end
  end

  % Pre-compute the pair potential
  Utab = GetUdir(ewcoeff, Lcut, 0.001);

  % Loop over all subdomains
  f = zeros(natom, 3);
  for i = 1:1:nbx(1),
    for j = 1:1:nbx(2),
      for k = 1:1:nbx(3),

        % Compute forces by the midpoint method
        LC(i,j,k) = SubCellFrc(LC(i,j,k), Utab, Lcut);

        % Now, put forces back together
        for m = 1:1:LC(i,j,k).natom,
          f(LC(i,j,k).anum(m),:) = f(LC(i,j,k).anum(m),:) + LC(i,j,k).frc(m,:);
        end
      end
    end
  end

end
