% pmeDirectTowerPlate: this function uses the "Tower and Plate" method to
%                      compute the direct space interactions for Smooth
%                      Particle Mesh Ewald calculations.
%
%   Arguments:
%   crdq:    the XYZ coordinates and charges for N atoms in an N x 4 matrix
%            (the number of atoms N is automatically detected)
%   excl:    the exclusions (not yet implemented, just feed the function a
%            scalar number as a placeholder for this argument)
%   gdim:    1 x 3 array, the unit cell dimensions (an orthorhombic unit
%            cell is assumed for the time being)
%   ewcoeff: the Ewald coefficient (I prefer to think in terms of the
%            Gaussian width, which is 0.5/ewcoeff, but for the purposes of
%            coding we work with this number)
%   Lcut:    the direct space cutoff (typical values are 8-9A)
%
%   In this simplified implementation of the Tower and Plate method, there
%   are no optimizations for trimming the import region or subdividing the
%   system into more towers and plates (which can greatly reduce the number
%   of distances that must be tested in order to find all relevant
%   interactions).  In Matlab, however, this implementation is about as
%   fast as anything.  Note that it is somewhat faster than the Midpoint
%   method, primarily because there are fewer conditional statements to
%   evaluate (in particular, there is no need to test the location of an
%   interaction's midpoint, just the distance between two particles).
%
%   Usage:
%   [f] = pmeDirectTowerPlate(crdq, excl, gdim, ewcoeff, Lcut) returns the
%     direct space forces on all atoms
%   [f,LC] = pmeDirectTowerPlate(...) returns the forces as well as the
%     data structure LC used to organize the atoms during the calculation.
function [f,LC] = pmeDirectTowerPlate(crdq, excl, gdim, ewcoeff, Lcut)

  % First, divide the cell into sub-blocks.  The sub-blocks must be at
  % least 1/2 Lcut tall, wide, and thick.  In the non-orthorhombic case,
  % we cannot go by distances along the lattice vectors alone; we must
  % instead compute the distance in cartesian coordinates.  Fortunately,
  % the inverse transformation matrix provides all the necessary numbers
  % very conveniently.
  szg = size(gdim);
  szg = szg(1,2);
  
  % If we have an orthorhombic unit cell, we can still make transformation
  % matrices for it.  Note that the 4th, 5th, and 6th entries of gdim are
  % assumed to be in radians by the time they reach this function
  % (arguments to pmeMain are given in degrees, but converted to radians
  % within that function).
  if (szg == 3),
    gdim(4:6) = pi/2;
  end
  [U,invU] = CompXfrm(gdim);

  % We use the Hessian normal form to compute the thickness of the box
  % (distance between box faces) in each of three dimensions.  Note that
  % thx, thy, and thz start life as vectors but then become scalars.
  % This is Matlab code, after all!  The origin of the unit cell is
  % assumed to be at (0,0,0) and xyz0 is the farthest corner.
  xyz0 = invU*ones(3,1);
  thx = cross(invU(:,2), invU(:,3));
  thx = abs(dot(thx/sqrt(sum(thx.^2)), xyz0));
  thy = cross(invU(:,1), invU(:,3));
  thy = abs(dot(thy/sqrt(sum(thy.^2)), xyz0));
  thz = cross(invU(:,1), invU(:,2));
  thz = abs(dot(thz/sqrt(sum(thz.^2)), xyz0));
  ncell = floor([thx thy thz]/Lcut);
  cellx = 1.0./ncell;

  % Pre-allocate the cell array LC
  natom = size(crdq);
  natom = natom(1,1);
  mxatm = ceil(natom/(ncell(1)*ncell(2)*ncell(3)));
  LEx.natom = 0;
  LEx.maxatom = 0;
  LEx.center = [ 0.0 0.0 0.0 ];
  LEx.crd = zeros(3,3);
  LEx.q = zeros(3,1);
  LEx.anum = zeros(3,1);
  LC(ncell(1), ncell(2), ncell(3)).natom = LEx;
  for i = 1:1:ncell(1),
    for j = 1:1:ncell(2),
      for k = 1:1:ncell(3),
        LC(i,j,k).natom = 0;
        LC(i,j,k).maxatom = mxatm;
        LC(i,j,k).center = invU*([ i-0.5 j-0.5 k-0.5 ].*cellx)';
        LC(i,j,k).crd = zeros(mxatm,3);
        LC(i,j,k).q = zeros(mxatm,1);
        LC(i,j,k).anum = zeros(mxatm,1);
      end
    end
  end
  
  % Convert all atoms into box space and assign them to cells.  Note that
  % bxcrd is going to remain a 3 x natom array, while crdq is an natom x 4
  % array.
  bxcrd = U*crdq(:,1:3)';
  bxcrd = bxcrd - floor(bxcrd);
  reimcrd = (invU*bxcrd)';
  for i = 1:1:3,
    bxcrd(i,:) = bxcrd(i,:)*ncell(i);
  end
  for i = 1:1:natom,
  
    % Determine which cell this atom sits in
    ci = floor(bxcrd(1,i))+1;
    cj = floor(bxcrd(2,i))+1;
    ck = floor(bxcrd(3,i))+1;
    
    % If we're running out of room, allocate some more
    ntt = LC(ci,cj,ck).natom + 1;
    ntmax = LC(ci,cj,ck).maxatom;
    if (ntt > ntmax),
      LC(ci,cj,ck).maxatom = ntmax + 16;
      LC(ci,cj,ck).crd(ntmax+1:ntmax+16,1:3) = 0.0;
      LC(ci,cj,ck).q(ntmax+1:ntmax+16,1) = 0.0;
      LC(ci,cj,ck).anum(ntmax+1:ntmax+16,1) = 0.0;
    end
    LC(ci,cj,ck).crd(ntt,:) = reimcrd(i,:);
    LC(ci,cj,ck).q(ntt,1) = crdq(i,4);
    LC(ci,cj,ck).anum(ntt,1) = i;
    LC(ci,cj,ck).natom = ntt;
  end

  % Compute the potential lookup table
  Utab = GetUdir(ewcoeff, Lcut, 0.01);
  
  % Now, loop over all blocks and compute interactions
  f = zeros(natom, 3);
  for i = 1:1:ncell(1),
    for j = 1:1:ncell(2),
      for k = 1:1:ncell(3),
        f = TowerPlateFrc(LC, i, j, k, ncell, invU, Lcut, Utab, f);
      end
    end
  end
end
