% pmeMain: this function wraps the PME direct space and reciprocal space
%          calculations.
%
%   Arguments:
%   crdq:  an N x 4 matrix containing the XYZ coordinates and charges of
%          each of N atoms (the number of atoms is automatically detected
%          in each of the various sub-procedures)
%   ng:    1 x 3 array, the size of the mesh
%   gdim:  1 x 3 array, the dimensions of the unit cell in Angstroms (the
%          cell is assumed to be orthorhombic), --OR--
%          1 x 6 array, the lengths of the lattice vectors in Angstroms
%          followed by the box Alpha, Beta, and Gamma angles in degrees
%          (the cell is assumed to be mono- or tri-clinic)
%   ordr:  the order of B-spline interpolation
%   Dtol:  the direct sum tolerance (set to negative to activate self image
%          force corrections)
%   Lcut:  the direct space cutoff
%
%   This function is meant as a pedagogical tool for those already familiar
%   with the basic procedure of computing electrostatics for a non-periodic
%   system of point charges.  As such, the "direct space" calculation that
%   looks and feels much like the typical point-charge to point-charge
%   interactions is done using a version of the "midpoint" method for speed
%   in Matlab but also to show how the method works, i.e. decomposing the
%   system into sub-blocks, importing to each sub-block all atoms whose
%   mutual interactions could possibly have midpoints within that
%   sub-block, and then computing all such interactions.  The reciprocal
%   space calculation returns only forces; calculation of the energy and
%   virial are not difficult once these concepts are understood.
%
%   Currently, this function does not consider exclusions between bonded
%   atoms.  This involves explicit corrections to the reciprocal space sum,
%   (corrections which are computed in direct space).  But, this procedure
%   does agree very closely with AMBER results for condensed systems of
%   ions, for example.
%
%   The entire Smooth Particle Mesh Ewald method is described in Essmann et
%   al., J. Chem. Phys. 103: 8577 (1995).  The terminology used in these
%   function m-files and most of the variable names follow that
%   publication.
%
%   Usage:
%   [frc] = pmeMain(crdq, ng, gdim, ordr, Dtol, Lcut) returns forces acting
%     on all particles
%   [frc,fdir,frec] = pmeMain(...) returns total forces as well as direct
%     and reciprocal space forces
function [frc,fdir,frec] = pmeMain(crdq, ng, gdim, ordr, Dtol, Lcut)

  % Check neutrality of the total charge in the box
  crdq = CheckNeutral(crdq);

  % If we are dealing with non-orthorhombic boxes, the box angles are
  % input in degrees, but we must convert them to radians.
  szg = size(gdim);
  szg = szg(1,2);
  if (szg == 6),
    gdim(4:6) = gdim(4:6)*pi/180.0;
  end

  % Compute the ewald coefficient.  There is first a check on whether Dtol
  % is negative, a flag which activates self-image force corrections.
  if (Dtol < 0.0),
    chkself = -1.0;
    Dtol = abs(Dtol);
  else
    chkself = 1.0;
  end
  ewcoeff = getalpha(Lcut, Dtol);
  
  % Compute the direct space sum.
  fdir = pmeDirectTowerPlate(crdq, [5], gdim, ewcoeff, Lcut);

  % Compute the reciprocal space sum.
  frec = pmeRecip(crdq, ng, gdim, ordr, chkself*ewcoeff);
  frec = ZeroForce(frec);
  
  frc = fdir + frec;

end
