% pmeNonOrthoConvBC: this function performs the convolution of the charge
%                    grid Q with the reciprocal space pair potential,
%                    defined by the grids B and C (notation taken from
%                    Essmann, JCP 103: 8577, 1995).  
%
%   Arguments:
%   Q:    the charge grid
%   gdim: the dimensions of the unit cell
%   ordr: the order of B-spline interpolation
%   S:    the width of Gaussian spread on each charge
%
%   B is an "adapter" grid needed to take care of the fact that we have
%   placed the charges onto Q using B-splines.  C folds together the
%   division by the squared mangitude of the frequency vector (essentially,
%   how far you are from (0,0,0) in k-space) and the convolution of Q with
%   a Gaussian of width S.
%
%   This function works for all unit cells regardless of the orthogonality
%   of any of the lattice vectors; the pmeOrthoConvBC.m function is merely
%   an optimized version of this function for use with rectangular unit
%   cells.
%
%   Note also that the reciprocal space pair potential can be visualized in
%   real space.  To facilitate this, the grids B and C can be returned.
%   Taking the inverse fourier transform ifftn(Q0.*B.*C), where Q0 is a
%   grid of zeros except for Q(1,1,1) = 332.0636, will produce the
%   reciprocal space pair potential in real space, in units of kcal/mol-e2. 
%
%   Usage:
%   [U] = pmeNonOrthoConvBC(Q, gdim, ordr, S) returns the electrostatic
%     potential due to the charge grid Q in the reciprocal space Smooth
%     Particle Mesh Ewald calculation
%   [U,Enrg,B,C] = pmeNonOrthoConvBC(...) returns the electrostatic
%     potential as well as the electrostatic energy Enrg and the B and C
%     grids used to convolute Q
function [U,Enrg,B,C] = pmeNonOrthoConvBC(Q, gdim, ordr, S)

  % Initialize the grids
  ng = size(Q);
  B = zeros(ng(1), ng(2), ng(3));
  C = zeros(ng(1), ng(2), ng(3));

  % Because we have non-orthogonal lattice vectors, we can only pre-tabulate
  % for B.
  szo = size(ordr);
  szo = szo(1,2);
  if (szo == 3),
    ordr1 = ordr(1);
    ordr2 = ordr(2);
    ordr3 = ordr(3);
  else
    ordr1 = ordr;
    ordr2 = ordr;
    ordr3 = ordr;
  end
  [bx,mx] = pmeNonOrthoTabBC(ng, 1, ordr1);
  [by,my] = pmeNonOrthoTabBC(ng, 2, ordr2);
  [bz,mz] = pmeNonOrthoTabBC(ng, 3, ordr3);

  % Take the charge mesh into frequency space.
  fQ = fftn(Q);
  
  % Omit any net charge on the system.
  fQ(1,1,1) = 0.0;

  % Compute the box transformation matrices and other prefactors
  [U,invU] = CompXfrm(gdim);

  % Transpose the rotation matrices to agree with AMBER... not sure
  % whether I should be doing this...
  U = U';
  invU = invU';
  tpS = (2.0*pi*S)^2;
  pivol = ng(1)*ng(2)*ng(3)/(pi*invU(1,1)*invU(2,2)*invU(3,3));
  for i = 1:1:ng(1),
    dx = mx(i);
    for j = 1:1:ng(2),
      dy = my(j);
      for k = 1:1:ng(3),
        dz = mz(k);
        mmx = U(1,1)*dx + U(1,2)*dy + U(1,3)*dz;
        mmy = U(2,1)*dx + U(2,2)*dy + U(2,3)*dz;
        mmz = U(3,1)*dx + U(3,2)*dy + U(3,3)*dz;
        mm2 = mmx*mmx + mmy*mmy + mmz*mmz;
        
        % The B mesh depends only on the size of the grid (even in
        % non-orthorhombic unit cells).
        B(i,j,k) = bx(i,1)*by(j,1)*bz(k,1);
        
        % The C mesh is 
        C(i,j,k) = pivol*exp(-tpS*mm2)/mm2;
      end
    end
  end
  C(1,1,1) = 0.0;

  Enrg = 0.5*sum(sum(sum((abs(fQ).^2).*B.*C)))/(ng(1)*ng(2)*ng(3));

  % The convolution theorem: f (convolution) g = IFFT(FFT(f).*FFT(g)).  In
  % this case, we get the electrostatic potential from the charge mesh Q
  % and the reciprocal space pair potential B.*C.
  fQBC = fQ.*B.*C;
  U = ifftn(fQBC);

return
