% pmeOrthoConvBC: this function performs the convolution of the charge grid
%                 Q with the reciprocal space pair potential, defined by
%                 the grids B and C (notation taken from Essmann, JCP 103:
%                 8577, 1995).  
%   Arguments:
%   Q:    the charge grid
%   gdim: the dimensions of the unit cell
%   ordr: the order of B-spline interpolation
%   S:    the width of Gaussian spread on each charge
%
%   B is an "adapter" grid needed to take care of the fact that we have
%   placed the charges onto Q using B-splines.  C folds together the
%   division by the squared mangitude of the frequency vector (essentially,
%   how far you are from (0,0,0) in k-space) and the convolution of Q with
%   a Gaussian of width S.
%
%   As the name implies, this function works for unit cells with orthogonal
%   lattice vectors.
%
%   Note also that the reciprocal space pair potential can be visualized in
%   real space.  To facilitate this, the grids B and C can be returned.
%   Taking the inverse fourier transform ifftn(Q0.*B.*C), where Q0 is a
%   grid of zeros except for Q(1,1,1) = 332.0636, will produce the
%   reciprocal space pair potential in real space, in units of kcal/mol-e2. 
%
%   Usage:
%   [U] = pmeOrthoConvBC(Q, gdim, ordr, S) returns the electrostatic
%     potential due to the charge grid Q in the reciprocal space Smooth
%     Particle Mesh Ewald calculation
%   [U,Enrg,B,C] = pmeOrthoConvBC(...) returns the electrostatic potential
%     as well as the electrostatic energy Enrg and, if desired, the B and C
%     grids used to convolute Q
function [U,Enrg,B,C] = pmeOrthoConvBC(Q, gdim, ordr, S)

  % Recall the dimensions of Q
  ng = size(Q);
  
  % Initialize the grids
  B = zeros(ng(1), ng(2), ng(3));
  C = zeros(ng(1), ng(2), ng(3));
  
  % Because we have orthogonal lattice vectors, we can pre-tabulate for
  % both B and C.
  szo = size(ordr);
  szo = szo(1,2);
  if (szo == 3),
    ordr1 = ordr(1);
    ordr2 = ordr(2);
    ordr3 = ordr(3);
  else
    ordr1 = ordr;
    ordr2 = ordr;
    ordr3 = ordr;
  end
  [bx,mx,ex] = pmeOrthoTabBC(ng, gdim, 1, ordr1, S);
  [by,my,ey] = pmeOrthoTabBC(ng, gdim, 2, ordr2, S);
  [bz,mz,ez] = pmeOrthoTabBC(ng, gdim, 3, ordr3, S);

  % Take the charge mesh into frequency space.
  fQ = fftn(Q);
  
  % Omit any net charge on the system.
  fQ(1,1,1) = 0.0;

  for i = 1:1:ng(1),
    dx = mx(i)^2;
    for j = 1:1:ng(2),
      dxy = dx + my(j)^2;
      for k = 1:1:ng(3),
        dxyz = dxy + mz(k)^2;

        % The B mesh depends only on the size of the grid (even in
        % non-orthorhombic unit cells).
        B(i,j,k) = bx(i,1)*by(j,1)*bz(k,1);

        % The sum of a series of convolutions of Q with different Gaussians
        % is equal to the convolution of Q with the sum of the Gaussians.
        C(i,j,k) = ex(i,1)*ey(j,1)*ez(k,1)/dxyz;
      end
    end
  end
  C(1,1,1) = 0.0;

  Enrg = 0.5*sum(sum(sum((abs(fQ).^2).*B.*C)))/(ng(1)*ng(2)*ng(3));
  
  % The convolution theorem: f (convolution) g = IFFT(FFT(f).*FFT(g)).  In
  % this case, we get the electrostatic potential from the charge mesh Q
  % and the reciprocal space pair potential B.*C.
  fQBC = fQ.*B.*C;
  U = ifftn(fQBC);

return
