% pmeOrthoTabBC: if we are working in an orthorhombic unit cell, there is a
%                lot of stuff that can be pre-tabulated for computing the
%                grids B and C.  This function wraps up all that
%                pre-computation.  See the function pmeOrthoConvBC.m for
%                context and more description on the grids B and C.
%
%   Arguments:
%   ng:    1 x 3 integer array, the dimensions of the grid
%   gdim:  1 x 3 real array, the physical dimensions of the unit cell in A
%   xyz:   scalar integer, values 1, 2, or 3 acceptable--the dimension for
%          pre-tabulation
%   ordr:  the order of interpolation
%   S:     the width of the Gaussian used for charge smoothing
%
%   Usage:
%   [bv,mv,ev] = pmeOrthoTabBC(ng, gdim, xyz, ordr, S) returns the
%     optimized prefactors for the B grid as bv, components of the
%     frequency vector as mv, and prefactors for the C grid as ev
function [bv,mv,ev] = pmeOrthoTabBC(ng, gdim, xyz, ordr, S)

  % Check input
  if (xyz ~= 1 && xyz ~= 2 && xyz ~= 3),
    fprintf('pmeOrthoTabBC: Error.  xyz dimension invalid.\n');
    return
  end

  % If this is the X dimension, we fold in the inverse volume factor
  % This only applies in the orthorhombic cell case.  In other unit
  % cells, the ev output is ignored.
  if (xyz == 1),
    pivol = ng(1)*ng(2)*ng(3)/(pi*gdim(1)*gdim(2)*gdim(3));
  end

  bv = LoadPrefac(ordr, ng(xyz));
  ev = zeros(ng(xyz),1);
  mv = zeros(ng(xyz),1);

  % C table
  tpS = 2.0*pi*S;
  for i = 1:1:ng(xyz),
    if (i-1 <= ng(xyz)/2),
      mv(i,1) = (i-1)/gdim(xyz);
    else
      mv(i,1) = (i-1-ng(xyz))/gdim(xyz);
    end
    if (xyz == 1),
      ev(i,1) = pivol*exp(-(tpS*mv(i))^2);
    else
      ev(i,1) = exp(-(tpS*mv(i))^2);
    end
  end
  
end
