% pmeRecip: this function wraps the reciprocal space part of the standard
%           Smooth Particle Mesh Ewald calculation.  For a thorough
%           description, see Essmann et al., J. Chem. Phys, 103: 8577
%           (1995).
%
%   Arguments:
%   crdq:    an N x 4 matrix containing the XYZ coordinates and charges of
%            each of N atoms (the number of atoms is automatically detected
%            in each of the various sub-procedures)
%   ng:      1 x 3 array, the size of the mesh
%   gdim:    1 x 3 array, the dimensions of the unit cell in Angstroms (the
%            cell is assumed to be orthorhombic), --OR--
%            1 x 6 array, the lengths of the lattice vectors in Angstroms
%            followed by the box Alpha, Beta, and Gamma angles in degrees
%            (the cell is assumed to be mono- or tri-clinic)
%   ordr:    the order of B-spline interpolation
%   ewcoeff: the Ewald coefficient (supply a negative value to activate
%            self-force corrections)
%
%   Usage:
%   [f] = pmeRecip(crdq, ng, gdim, ordr, ewcoeff) returns the reciprocal
%     space forces on all atoms in crdq
%   [f,Q,U] = pmeRecip(...) returns the forces, charge mesh, and
%     electrostatic potential from the reciprocal space calculation.
function [f,Q,U] = pmeRecip(crdq, ng, gdim, ordr, ewcoeff)

  % Parameters!  The spread of the Gaussian charge smoothing function is
  % inversely related to the Ewald coefficient.
  if (ewcoeff < 0.0),
    chkself = 1;
    ewcoeff = abs(ewcoeff);
  else
    chkself = 0;
  end
  S = 0.5/ewcoeff;

  % Check that the order is an integer greater than or equal to 3
  szo = size(ordr);
  szo = szo(1,2);
  for i = 1:1:szo,
    dord = ordr(i) - round(ordr(i));
    if (abs(dord) > 1.0e-7),
      fprintf('pmeRecip: Warning.  Interpolation order must be an ');
      fprintf('integer.  Rounding.\n');  
      ordr(i) = ordr(i) - dord;
    end
    if (ordr(i) < 3),
      fprintf('pmeRecip: Error.  Interpolation order %d too low.\n', ordr);
      return
    end
  end
  if (szo == 3),
    ordr1 = ordr(1);
    ordr2 = ordr(2);
    ordr3 = ordr(3);
  else
    ordr1 = ordr;
    ordr2 = ordr;
    ordr3 = ordr;
  end

  % Detect whether we have an orthorhombic unit cell or not, based on
  % whether gdim has just three indices (which are then assumed to be the
  % box lengths) or six (which are then assumed to be the box lengths
  % followed by the box angles).
  szg = size(gdim); 
  szg = szg(1,2);

  % Compute splines and positions on grid
  if (szg == 3),
    [ SPx dSPx iqmap ] = splcof(crdq, ng, gdim, ordr1, 1);
    [ SPy dSPy jqmap ] = splcof(crdq, ng, gdim, ordr2, 2);
    [ SPz dSPz kqmap ] = splcof(crdq, ng, gdim, ordr3, 3);
  else
    bxcrdq = Crd2BoxSpace(crdq, gdim);

    [ SPx dSPx iqmap ] = splcof(bxcrdq, ng, [1.0 1.0 1.0], ordr1, 1);
    [ SPy dSPy jqmap ] = splcof(bxcrdq, ng, [1.0 1.0 1.0], ordr2, 2);
    [ SPz dSPz kqmap ] = splcof(bxcrdq, ng, [1.0 1.0 1.0], ordr3, 3);
  end

  % Map the charges to the grid Q
  Q = MapCharges(ng, SPx, SPy, SPz, iqmap, jqmap, kqmap);

  % Convolute Q with the reciprocal space pair potential to get the
  % electrostatic potential grid U
  if (szg == 3),
    [U,Enrg,B,C] = pmeOrthoConvBC(Q, gdim, ordr, S);
  else
    [U,Enrg,B,C] = pmeNonOrthoConvBC(Q, gdim, ordr, S);
  end

  % Interpolate forces on each atom from U
  f = IFrc(SPx, SPy, SPz, dSPx, dSPy, dSPz, iqmap, jqmap, kqmap, U, gdim);

  % Apply self-force correction
  if (chkself == 1),
    fc = EvalSelfFrc(crdq, B, C, ordr, gdim);
    f = f + fc;
  end
end
