% splcof: compute spline coefficients for B-spline interpolation of
%         charges.  
%
%   Arguments:
%   crdq:  an N x 4 matrix containing the XYZ coordinates and charges of
%          each of N atoms (the number of atoms is automatically detected
%          in each of the various sub-procedures)
%   ng:    1 x 3 integer array, the dimensions of the grid
%   gdim:  1 x 3 real array, the physical dimensions of the unit cell in A
%   ordr:  the order of interpolation
%   xyz:   scalar integer, values 1, 2, or 3 acceptable--the dimension for
%          pre-tabulation
%
%   Note that this function is highly optimized for fourth-order
%   interpolation, which is used almost exclusively in molecular dynamics
%   simulations.  Third order interpolation can be used in theory, but
%   there is a very high price in the accuracy of the calculation.  Any
%   lower order of interpolation cannot yield forces because B-splines are
%   only n-2 times continuously differentiable.  Higher orders of
%   interpolation can be used to obtain comparable accuracy with a coarser
%   mesh or a shorter direct space cutoff Lcut (smaller values of Lcut,
%   ceteris paribus, imply a steeper reciprocal space pair potential that
%   will not map as accurately any given mesh density).
%
%   Usage:
%   [SP,dSP,qmap] = splcof(crdq, ng, gdim, ordr, xyz) returns the B-Spline
%     coefficients, derivatives, and mapping indices for the dimesion xyz
function [SP,dSP,qmap] = splcof(crdq, ng, gdim, ordr, xyz)

  % Check input
  if (xyz ~= 1 && xyz ~= 2 && xyz ~= 3),
    fprintf('splcof: Error.  xyz dimension invalid.\n');
  end

  % Find the number of atoms
  natom = size(crdq);
  natom = natom(1,1);

  % Convert to scaled fractional coordinates
  u = (ng(xyz)/gdim(xyz))*crdq(:,xyz);
  uc = ceil(u);
  
  % Initialize arrays for B-spline coefficients and charge map locations
  SP = zeros(natom, ordr);
  dSP = zeros(natom, ordr);
  qmap = zeros(natom, ordr);

  % Optimized code for 4th order interpolation
  if (ordr == 4),

    % In this case, w is non-integer part of the fractional coordinate.
    % We first compute third-order B-spline coefficients for this
    % particular w.  Note that the fact that B-splines form a smooth
    % partition of unity (all the spline coefficients are guaranteed to
    % add to 1) is used to determine SP(h,2), thus saving a few
    % operations and improving the numerical stability (otherwise, you
    % might end up mapping slightly more or less charge to the grid than
    % you're supposed to, although Matlab's double precision arithmetic
    % would be fine either way).
    w = u(:,1)-uc(:,1)+1;
    
    SP(:,3) = 0.5.*w.*w;
    SP(:,1) = 0.5*(1.0-w).*(1.0-w);
    SP(:,2) = 1.0-SP(:,1)-SP(:,3);

    % We now obtain the fourth-order spline derivatives from the third
    % order spline coefficients, using the definition of the B-spline
    % derivative and exploiting the fact that the third order B-spline is
    % zero outside of the range (0, 3).
    dSP(:,1) = -SP(:,1);
    dSP(:,2) = SP(:,1) - SP(:,2);
    dSP(:,3) = SP(:,2) - SP(:,3);
    dSP(:,4) = SP(:,3);
      
    % Finally, we obtain the fourth order spline coefficients, again
    % using the fact that B-splines form a smooth partition of unity.
    SP(:,4) = w.*SP(:,3)/3.0;
    SP(:,3) = ((w+1.0).*SP(:,2) + (3.0-w).*SP(:,3))/3.0;
    SP(:,1) = (1.0-w).*SP(:,1)/3.0;
    SP(:,2) = 1.0-SP(:,1)-SP(:,3)-SP(:,4);
  else
    du = u(:,1) - ceil(u(:,1)) + 1.0;
    for i = ordr-1:-1:0,

      % Some non-optimized code for general interpolation orders... in
      % C this would take several times longer.
      ip = i + 1;
      SP(:,ip) = bsplnV(du, ordr);
      dSP(:,ip) = dbsplnV(du, ordr);
      du = du + 1;
    end
  end
    
  % Now that bit about the charge mapping region...
  qmap = zeros(natom, ordr);
  for i = 0:1:ordr-1,
    ip = i + 1;
    qmap(:,ip) = uc(:,1) - ordr + i;
    qmap(:,ip) = qmap(:,ip) - ng(xyz)*floor(qmap(:,ip)/ng(xyz));
  end

  % Add one since we are dealing with Matlab, not C arrays
  qmap = qmap + 1;

  % If these spline coefficients pertain to the X dimension, we just fold
  % in the charge of the atom here.  It will be carried through the
  % calculation onto the charge grid.
  if (xyz == 1),
    for i = 1:1:ordr,
      SP(:,i) = SP(:,i).*crdq(:,4);
      dSP(:,i) = dSP(:,i).*crdq(:,4);
    end
  end

return
